import epics
import time
import json
import os
import matplotlib.cm
from io import BytesIO
from PIL import Image
from git import Repo
from shutil import rmtree
from glob import glob
from numpy import nan, linspace, array, zeros
from http.server import BaseHTTPRequestHandler,HTTPServer
from epicsarchiver import ArchiverAppliance
from threading import Thread, Event
from queue import Queue

PORT_NUMBER = 8080
EPICS_SERVER = '10.0.16.88'
os.environ['EPICS_CA_ADDR_LIST'] = EPICS_SERVER
os.environ['EPICS_CA_AUTO_ADDR_LIST'] = 'NO'
ARCHIVER_SERVER = 'archiver-01.tn.esss.lu.se'
archiver = ArchiverAppliance(ARCHIVER_SERVER)
pv_repo='https://bitbucket.org/europeanspallationsource/isrc-lebt-pv-list.git'
stop_signal = Event()
stop_signal.clear()

try:
    Repo.clone_from(pv_repo, './repo')
    pvfiles=glob('./repo/*.txt')
except:
    print('PV Reopsitory not available')
    quit()

pvlist=[]
for filename in pvfiles:
    sheet = filename.split('/')[2].split('.')[0]
    data=open(filename,'r')
    pvlist+=data.read().splitlines()
    data.close()
rmtree('./repo')
pvlist.sort()

global_dict={}
for PV in pvlist:
    try:
        sheet=PV.split(':')[0]
        column=PV.split(':')[1]
        if sheet not in global_dict.keys():
            global_dict[sheet]={}
        if column not in global_dict[sheet].keys():
            global_dict[sheet][column]={}
        global_dict[sheet][column][PV]=False
    except:
        None

with open('/var/www/data/getData','w') as datafile:
    json.dump(global_dict, datafile)

class epicsQuery(Thread):
    def __init__(self, stop_signal):
        Thread.__init__(self)
        self.stop_signal = stop_signal

    def run(self):
        print('Connect to EPICS')
        publicpvs={}
        publicpvs['ISrc-010:ISS-Magtr:PulsHLvlS']=epics.PV('ISrc-010:ISS-Magtr:PulsHLvlS', auto_monitor=True) # Magnetron Power
        publicpvs['ISrc-010:ISS-Magtr:Setup.B6']=epics.PV('ISrc-010:ISS-Magtr:Setup.B6', auto_monitor=True) # Magnetron on/off
        publicpvs['ISrc-010:PwrC-RepPS-01:PwrR']=epics.PV('ISrc-010:PwrC-RepPS-01:PwrR', auto_monitor=True) # Repeller on/off
        publicpvs['ISrc-010:PwrC-RepPS-01:VolR']=epics.PV('ISrc-010:PwrC-RepPS-01:VolR', auto_monitor=True) # Repeller voltage
        publicpvs['LEBT-010:PwrC-RepPS-01:PwrR']=epics.PV('LEBT-010:PwrC-RepPS-01:PwrR', auto_monitor=True) # Repeller on/off 
        publicpvs['LEBT-010:PwrC-RepPS-01:VolR']=epics.PV('LEBT-010:PwrC-RepPS-01:VolR', auto_monitor=True) # Repeller voltage
        publicpvs['ISrc-010:ISS-HVPS:PwrR']=epics.PV('ISrc-010:ISS-HVPS:PwrR', auto_monitor=True) # HV on/off
        publicpvs['ISrc-010:ISS-HVPS:VolR']=epics.PV('ISrc-010:ISS-HVPS:VolR', auto_monitor=True) # HV voltage
        publicpvs['ISrc-010:TS-EVG-01:Mxc1-Frequency-RB']=epics.PV('ISrc-010:TS-EVG-01:Mxc1-Frequency-RB', auto_monitor=True) # Frequency
        publicpvs['ISrc-010:ISS-EVR-Magtr:Pul0-Width-RB']=epics.PV('ISrc-010:ISS-EVR-Magtr:Pul0-Width-RB', auto_monitor=True) # Pulse length
        publicpvs['ISrc-010:Vac-VVMC-01100:FlwR']=epics.PV('ISrc-010:Vac-VVMC-01100:FlwR', auto_monitor=True) # H2 flow
        publicpvs['LEBT-010:Vac-VVMC-01100:FlwR']=epics.PV('LEBT-010:Vac-VVMC-01100:FlwR', auto_monitor=True) # N2 flow
        publicpvs['ISrc-010:Vac-VVA-01100:OpenR']=epics.PV('ISrc-010:Vac-VVA-01100:OpenR', auto_monitor=True) # H2 on/off
        publicpvs['LEBT-010:Vac-VVA-01100:OpenR']=epics.PV('LEBT-010:Vac-VVA-01100:OpenR', auto_monitor=True) # N2 on/off
        publicpvs['LEBT-010:Vac-VVS-20000:ClosedR']=epics.PV('LEBT-010:Vac-VVS-20000:ClosedR', auto_monitor=True) # Vacuum status
        publicpvs['ISrc-010:PwrC-CoilPS-01:CurR']=epics.PV('ISrc-010:PwrC-CoilPS-01:CurR', auto_monitor=True) # Coil 1 current
        publicpvs['ISrc-010:PwrC-CoilPS-02:CurR']=epics.PV('ISrc-010:PwrC-CoilPS-02:CurR', auto_monitor=True) # Coil 2 current
        publicpvs['ISrc-010:PwrC-CoilPS-03:CurR']=epics.PV('ISrc-010:PwrC-CoilPS-03:CurR', auto_monitor=True) # Coil 3 current
        publicpvs['ISrc-010:PwrC-CoilPS-01:PwrR']=epics.PV('ISrc-010:PwrC-CoilPS-01:PwrR', auto_monitor=True) # Coil 1 on/off
        publicpvs['ISrc-010:PwrC-CoilPS-02:PwrR']=epics.PV('ISrc-010:PwrC-CoilPS-02:PwrR', auto_monitor=True) # Coil 2 on/off
        publicpvs['ISrc-010:PwrC-CoilPS-03:PwrR']=epics.PV('ISrc-010:PwrC-CoilPS-03:PwrR', auto_monitor=True) # Coil 3 on/off
        publicpvs['LEBT-010:PwrC-SolPS-01:PwrR']=epics.PV('LEBT-010:PwrC-SolPS-01:PwrR', auto_monitor=True) # Sol 1 on/off
        publicpvs['LEBT-010:PwrC-SolPS-01:CurR']=epics.PV('LEBT-010:PwrC-SolPS-01:CurR', auto_monitor=True) # Sol 1 field
        publicpvs['LEBT-010:PwrC-SolPS-02:PwrR']=epics.PV('LEBT-010:PwrC-SolPS-02:PwrR', auto_monitor=True) # Sol 2 on/off
        publicpvs['LEBT-010:PwrC-SolPS-02:CurR']=epics.PV('LEBT-010:PwrC-SolPS-02:CurR', auto_monitor=True) # Sol 2 field
        publicpvs['LEBT-010:PwrC-PSCH-01:PwrR']=epics.PV('LEBT-010:PwrC-PSCH-01:PwrR', auto_monitor=True) # H1 steerer on/off
        publicpvs['LEBT-010:PwrC-PSCH-01:CurR']=epics.PV('LEBT-010:PwrC-PSCH-01:CurR', auto_monitor=True) # H1 steerer field
        publicpvs['LEBT-010:PwrC-PSCH-02:PwrR']=epics.PV('LEBT-010:PwrC-PSCH-02:PwrR', auto_monitor=True) # H2 steerer on/off
        publicpvs['LEBT-010:PwrC-PSCH-02:CurR']=epics.PV('LEBT-010:PwrC-PSCH-02:CurR', auto_monitor=True) # H2 steerer field
        publicpvs['LEBT-010:PwrC-PSCV-01:PwrR']=epics.PV('LEBT-010:PwrC-PSCV-01:PwrR', auto_monitor=True) # V1 steerer on/off
        publicpvs['LEBT-010:PwrC-PSCV-01:CurR']=epics.PV('LEBT-010:PwrC-PSCV-01:CurR', auto_monitor=True) # V1 steerer field
        publicpvs['LEBT-010:PwrC-PSCV-02:PwrR']=epics.PV('LEBT-010:PwrC-PSCV-02:PwrR', auto_monitor=True) # V2 steerer on/off
        publicpvs['LEBT-010:PwrC-PSCV-02:CurR']=epics.PV('LEBT-010:PwrC-PSCV-02:CurR', auto_monitor=True) # V2 steerer field
        publicpvs['ISrc-010:ISS-EVR-Magtr:Pul1-Width-SP']=epics.PV('ISrc-010:ISS-EVR-Magtr:Pul1-Width-SP', auto_monitor=True) # Chopper Pulse Width
        publicpvs['NSO-LCR:Ops:Msg']=epics.PV('NSO-LCR:Ops:Msg', auto_monitor=True) # Control room messages
        publicpvs['NSO-LCR:Ops:SID']=epics.PV('NSO-LCR:Ops:SID', auto_monitor=True) # Shift ID
        publicpvs['NSO-LCR:Ops:MSL']=epics.PV('NSO-LCR:Ops:MSL', auto_monitor=True) # Shift Leader
        publicpvs['ISrc-010:PBI-BCM-001:AI4-Compressed']=epics.PV('ISrc-010:PBI-BCM-001:AI4-Compressed', auto_monitor=True) # IS BCM
        publicpvs['LEBT-010:PBI-BCM-001:AI5-Compressed']=epics.PV('LEBT-010:PBI-BCM-001:AI5-Compressed', auto_monitor=True) # LEBT BCM
        publicpvs['LEBT-020:PBI-FC-001:AMC31-AOI11-ArrayData']=epics.PV('LEBT-020:PBI-FC-001:AMC31-AOI11-ArrayData', auto_monitor=True) # LEBT FC
        publicpvs['LEBT-020:PBI-FC-001:AMC31-AOI11-StartR']=epics.PV('LEBT-020:PBI-FC-001:AMC31-AOI11-StartR', auto_monitor=True) # LEBT FC Start
        publicpvs['LEBT-020:PBI-FC-001:AMC31-AOI11-SizeR']=epics.PV('LEBT-020:PBI-FC-001:AMC31-AOI11-SizeR', auto_monitor=True) # LEBT FC Size
        publicpvs['LEBT-020:PBI-FC-001:MCU08-PositionR']=epics.PV('LEBT-020:PBI-FC-001:MCU08-PositionR', auto_monitor=True) # FC Position
        publicpvs['ISrc-010:ISS-EVR-Magtr:Event-14-Cnt-I']=epics.PV('ISrc-010:ISS-EVR-Magtr:Event-14-Cnt-I', auto_monitor=True) # Pulse Number
        
        interlocks={}
        interlocks['ISrc-010:ISS-Intf-HMI:ComIocOK']=epics.PV('ISrc-010:ISS-Intf-HMI:ComIocOK', auto_monitor=True)
        interlocks['ISrc-010:ISS-Intf-PSS:HVPermitR']=epics.PV('ISrc-010:ISS-Intf-PSS:HVPermitR', auto_monitor=True)
        interlocks['ISrc-010:ISS-HVPf-Res:CloseR']=epics.PV('ISrc-010:ISS-HVPf-Res:CloseR', auto_monitor=True)
        interlocks['ISrc-010:ISS-HVPf-Doors:CloseR']=epics.PV('ISrc-010:ISS-HVPf-Doors:CloseR', auto_monitor=True)
        interlocks['ISrc-010:ISS-Intf-Vac:VacPermit']=epics.PV('ISrc-010:ISS-Intf-Vac:VacPermit', auto_monitor=True)
        interlocks['LEBT-010:PBI-FC-001:WtrFlowOK']=epics.PV('LEBT-010:PBI-FC-001:WtrFlowOK', auto_monitor=True)
        interlocks['LEBT-010:PBI-EMU-001:VerWtrFlowOK']=epics.PV('LEBT-010:PBI-EMU-001:VerWtrFlowOK', auto_monitor=True)
        interlocks['LEBT-010:PBI-EMU-001:HorWtrFlowOK']=epics.PV('LEBT-010:PBI-EMU-001:HorWtrFlowOK', auto_monitor=True)
        interlocks['ISrc-010:PwrC-HVIT-01:LidOpenR']=epics.PV('ISrc-010:PwrC-HVIT-01:LidOpenR', auto_monitor=True)
        interlocks['ISrc-010:PwrC-HVIT-01:OilTempOK']=epics.PV('ISrc-010:PwrC-HVIT-01:OilTempOK', auto_monitor=True)
        interlocks['ISrc-010:ISS-HVPS:PwrR.SEVR']=epics.PV('ISrc-010:ISS-HVPS:PwrR.SEVR', auto_monitor=True)
        interlocks['ISrc-010:ISS-Intf-HMI:ESTOP_MEMO']=epics.PV('ISrc-010:ISS-Intf-HMI:ESTOP_MEMO', auto_monitor=True)
        interlocks['ISrc-010:ISS-HVPS:ITLckEnCmdR']=epics.PV('ISrc-010:ISS-HVPS:ITLckEnCmdR', auto_monitor=True)
        interlocks['ISrc-010:ISS-Intf-HMI:EVENT_B_MEMO']=epics.PV('ISrc-010:ISS-Intf-HMI:EVENT_B_MEMO', auto_monitor=True)

        instruments={}
        instruments['LEBT-010:PBI-NPM-001:HCAMOBSIMAGE-ArrayData']=epics.PV('LEBT-010:PBI-NPM-001:HCAMOBSIMAGE-ArrayData', auto_monitor=True)
        instruments['LEBT-010:PBI-NPM-002:VCAMOBSIMAGE-ArrayData']=epics.PV('LEBT-010:PBI-NPM-002:VCAMOBSIMAGE-ArrayData', auto_monitor=True)
        instruments['LEBT-010:PBI-NPM-001:HCAMFITS-Y_RBV']=epics.PV('LEBT-010:PBI-NPM-001:HCAMFITS-Y_RBV', auto_monitor=True)
        instruments['LEBT-010:PBI-NPM-002:VCAMFITS-Y_RBV']=epics.PV('LEBT-010:PBI-NPM-002:VCAMFITS-Y_RBV', auto_monitor=True)
        instruments['LEBT-010:PBI-NPM-001:HCAM-SCALEFACT']=epics.PV('LEBT-010:PBI-NPM-001:HCAM-SCALEFACT', auto_monitor=True)
        instruments['LEBT-010:PBI-NPM-002:VCAM-SCALEFACT']=epics.PV('LEBT-010:PBI-NPM-002:VCAM-SCALEFACT', auto_monitor=True)
        instruments['ISrc-010:PBI-BCM-001:AI4-Compressed']=publicpvs['ISrc-010:PBI-BCM-001:AI4-Compressed']
        instruments['LEBT-010:PBI-BCM-001:AI5-Compressed']=publicpvs['LEBT-010:PBI-BCM-001:AI5-Compressed']
        instruments['LEBT-020:PBI-FC-001:AMC31-AOI11-ArrayData']=publicpvs['LEBT-020:PBI-FC-001:AMC31-AOI11-ArrayData']
        instruments['LEBT-020:PBI-FC-001:AMC31-AOI11-StartR']=publicpvs['LEBT-020:PBI-FC-001:AMC31-AOI11-StartR']
        instruments['LEBT-020:PBI-FC-001:AMC31-AOI11-SizeR']=publicpvs['LEBT-020:PBI-FC-001:AMC31-AOI11-SizeR']
        instruments['LEBT-020:PBI-FC-001:MCU08-PositionR']=publicpvs['LEBT-020:PBI-FC-001:MCU08-PositionR']
        print('Done')
        while(not self.stop_signal.isSet()):
            publicpvs_return_dict={}
            for pv in publicpvs:
                publicpvs_return_dict[pv]={}
                if publicpvs[pv].connected:
                    publicpvs_return_dict[pv]['units']=publicpvs[pv].units
                    if pv=='ISrc-010:PBI-BCM-001:AI4-Compressed':
                        y_data_array=publicpvs[pv].value
                        y_data=list(y_data_array)
                        x_data_array=linspace(0,12.8,len(y_data))
                        x_data=list(x_data_array)
                        publicpvs_return_dict[pv]['value']=list(zip(x_data,y_data))
                        publicpvs_return_dict['pulse']={}
                        publicpvs_return_dict['max_curr']={}
                        publicpvs_return_dict['pulse']['units']='ms'
                        publicpvs_return_dict['max_curr']['units']='mA'
                        if y_data_array.max()<1:
                            publicpvs_return_dict['beam']=0
                            publicpvs_return_dict['pulse']['value']='0'
                            publicpvs_return_dict['max_curr']['value']='0'
                        else:
                            publicpvs_return_dict['beam']=1
                            threshold=max(y_data_array)*0.25
                            publicpvs_return_dict['pulse']['value']=round(x_data_array[y_data_array>threshold].max()-x_data_array[y_data_array>threshold].min(),2)
                            publicpvs_return_dict['max_curr']['value']=round(max(y_data_array),2)
                        continue
                    if pv=='LEBT-010:PBI-BCM-001:AI5-Compressed':
                        y_data=list(publicpvs[pv].value)
                        x_data=list(linspace(0,12.8,len(y_data)))
                        publicpvs_return_dict[pv]['value']=list(zip(x_data,y_data))
                        continue
                    if pv=='LEBT-020:PBI-FC-001:AMC31-AOI11-ArrayData':
                        fc_start = publicpvs['LEBT-020:PBI-FC-001:AMC31-AOI11-StartR'].value
                        fc_size = publicpvs['LEBT-020:PBI-FC-001:AMC31-AOI11-SizeR'].value
                        fc_resizer = int(fc_size/200)
                        y_data=list(publicpvs[pv].value[::fc_resizer])
                        x_data=list(linspace(fc_start/10000,(fc_start+fc_size)/10000,len(y_data)))
                        publicpvs_return_dict[pv]['value']=list(zip(x_data,y_data))
                        continue
                    try:
                        publicpvs_return_dict[pv]['value']=round(publicpvs[pv].value,3)
                    except:
                        publicpvs_return_dict[pv]['value']=publicpvs[pv].value
                else:
                    publicpvs_return_dict[pv]['units']='n.c.'
                    publicpvs_return_dict[pv]['value']=''
            try:
                publicpvs_return_dict['ISrc-010:TS-EVG-01:Mxc1-Frequency-RB']['value']=int(publicpvs_return_dict['ISrc-010:TS-EVG-01:Mxc1-Frequency-RB']['value'])
                publicpvs_return_dict['ISrc-010:ISS-EVR-Magtr:Pul0-Width-RB']['value']=int(publicpvs_return_dict['ISrc-010:ISS-EVR-Magtr:Pul0-Width-RB']['value'])
            except:
                publicpvs_return_dict['ISrc-010:TS-EVG-01:Mxc1-Frequency-RB']['value']=''
                publicpvs_return_dict['ISrc-010:ISS-EVR-Magtr:Pul0-Width-RB']['value']=''

            with open('/var/www/data/getPublicData','w') as datafile:
                json.dump(publicpvs_return_dict, datafile)

            interlocks_return_dict={}
            for pv in interlocks:
                interlocks_return_dict[pv]={}
                if interlocks[pv].connected:
                    interlocks_return_dict[pv]['units']=interlocks[pv].units
                    try:
                        interlocks_return_dict[pv]['value']=round(interlocks[pv].value,3)
                    except:
                        interlocks_return_dict[pv]['value']=interlocks[pv].value
                else:
                    interlocks_return_dict[pv]['units']='n.c.'
                    interlocks_return_dict[pv]['value']=''
            with open('/var/www/data/getInterlocks','w') as datafile:
                json.dump(interlocks_return_dict, datafile)

            colormap=matplotlib.cm.get_cmap('jet')
            try:
                imarray = (colormap(instruments['LEBT-010:PBI-NPM-002:VCAMOBSIMAGE-ArrayData'].value.reshape((618,824)))*255).astype('uint8')
            except:
                imarray = colormap(zeros((618,824))).astype('uint8')
            im = Image.fromarray(imarray)
            im.save('/var/www/data/npm-vimage.png', format='png')

            colormap=matplotlib.cm.get_cmap('jet')
            try:
                imarray = (colormap(instruments['LEBT-010:PBI-NPM-001:HCAMOBSIMAGE-ArrayData'].value.reshape((618,824)))*255).astype('uint8')
            except:
                imarray = colormap(zeros((618,824))).astype('uint8')
            im = Image.fromarray(imarray).convert('RGBA')
            im.save('/var/www/data/npm-himage.png', format='png')

            instruments_return_dict={}
            for pv in instruments:
                instruments_return_dict[pv]={}
                if instruments[pv].connected:
                    instruments_return_dict[pv]['units']=instruments[pv].units
                    if pv=='ISrc-010:PBI-BCM-001:AI4-Compressed':
                        y_data=list(instruments[pv].value)
                        x_data=list(linspace(0,12.8,len(y_data)))
                        instruments_return_dict[pv]['value']=list(zip(x_data,y_data))
                        continue
                    if pv=='LEBT-010:PBI-BCM-001:AI5-Compressed':
                        y_data=list(instruments[pv].value)
                        x_data=list(linspace(0,12.8,len(y_data)))
                        instruments_return_dict[pv]['value']=list(zip(x_data,y_data))
                        continue
                    if pv=='LEBT-020:PBI-FC-001:AMC31-AOI11-ArrayData':
                        fc_start = instruments['LEBT-020:PBI-FC-001:AMC31-AOI11-StartR'].value
                        fc_size = instruments['LEBT-020:PBI-FC-001:AMC31-AOI11-SizeR'].value
                        fc_resizer = int(fc_size/200)
                        y_data=list(instruments[pv].value[::fc_resizer])
                        x_data=list(linspace(fc_start/10000,(fc_start+fc_size)/10000,len(y_data)))
                        instruments_return_dict[pv]['value']=list(zip(x_data,y_data))
                        continue
                    if pv=='LEBT-010:PBI-NPM-001:HCAMFITS-Y_RBV':
                        y_data=list(instruments[pv].value)
                        x_size=len(y_data)*instruments['LEBT-010:PBI-NPM-001:HCAM-SCALEFACT'].value/1000
                        x_data=list(linspace(-x_size/2,x_size/2,len(y_data)))
                        instruments_return_dict[pv]['value']=list(zip(x_data,y_data))
                        continue
                    if pv=='LEBT-010:PBI-NPM-002:VCAMFITS-Y_RBV':
                        y_data=list(instruments[pv].value)
                        x_size=len(y_data)*instruments['LEBT-010:PBI-NPM-002:VCAM-SCALEFACT'].value/1000
                        x_data=list(linspace(-x_size/2,x_size/2,len(y_data)))
                        instruments_return_dict[pv]['value']=list(zip(x_data,y_data))
                        continue
                    if pv=='LEBT-010:PBI-NPM-001:HCAMOBSIMAGE-ArrayData' or pv=='LEBT-010:PBI-NPM-002:VCAMOBSIMAGE-ArrayData':
                        continue
                    try:
                        instruments_return_dict[pv]['value']=round(instruments[pv].value,3)
                    except:
                        instruments_return_dict[pv]['value']=instruments[pv].value
                else:
                    instruments_return_dict[pv]['units']='n.c.'
                    instruments_return_dict[pv]['value']=''
            with open('/var/www/data/getInstruments','w') as datafile:
                json.dump(instruments_return_dict, datafile)

            time.sleep(1)

        for pv in publicpvs:
            publicpvs[pv].clear_auto_monitor()
            publicpvs[pv].disconnect()
        for pv in interlocks:
            interlocks[pv].clear_auto_monitor()
            interlocks[pv].disconnect()
        for pv in instruments:
            instruments[pv].clear_auto_monitor()
            instruments[pv].disconnect()


class myHandler(BaseHTTPRequestHandler):
    def do_GET(self):
        self.send_response(200)
        path = list(filter(None,self.path.split('/')))
        if len(path) == 2:
            if path[0] == 'search':
                returnlist=''
                self.send_header('Content-type','application/json')
                self.send_header('Access-Control-Allow-Origin', '*')
                self.send_header('Access-Control-Allow-Headers', 'Authorization, Content-Type')
                self.end_headers()
                wordlist=path[1].split("%20")
                if len(wordlist)<=1 and len(wordlist[0])<2:
                    returnlist=''
                    return
                result=pvlist
                for word in wordlist:
                    result=[row for row in result if word.lower() in row.lower()]
#                if len(result)>500:
#                     returnlist='List too long, please refine your research'
#                else:
                if len(result)==0:
                    returnlist='No PV found.'
                else:
                    for pv in result:
                        returnlist+='<a href="#" onclick="searchPV(\''+pv+'\')">'+pv+'</a>'
                        returnlist+='<br>'
                self.wfile.write(bytes(json.dumps({'pvs':returnlist}), "utf-8"))
                return

            if path[0] == 'getPVHTML':
                self.send_header('Content-type','application/json')
                self.send_header('Access-Control-Allow-Origin', '*')
                self.send_header('Access-Control-Allow-Headers', 'Authorization, Content-Type')
                self.end_headers()
                pvinfo={}
                try:
                    myPV=epics.PV(path[1])
                    pvinfo['info']=myPV.info.replace('\n','<br>')
                    myPV.clear_auto_monitor()
                    myPV.disconnect()
                    archiver_stat = ''
                    archiver_dict=archiver.get_pv_status(pv=path[1])[0]
                    for i in archiver_dict:
                        archiver_stat = archiver_stat + i + ' = '+ archiver_dict[i]+'\n'
                    pvinfo['archiver']=archiver_stat.replace('\n','<br>')
                    pvinfo['live_view']='http://live.pos.esss.lu.se/live_view?PV='+path[1]
                    if archiver_dict['status'] == 'Being archived':
                        pvinfo['link']='<a target="_blank" href="http://plot.pos.esss.lu.se/retrieval/ui/viewer/archViewer.html?pv='+path[1]+'">See archived data</a>'
                    self.wfile.write(bytes(json.dumps(pvinfo), "utf-8"))
                    return
                except:
                    pvinfo['info']="The PV seems disconnected"
                    pvinfo['archiver']=''
                    self.wfile.write(bytes(json.dumps(pvinfo), "utf-8"))
                    return

            if path[0] == 'getPV':
                self.send_header('Content-type','application/json')
                self.send_header('Access-Control-Allow-Origin', '*')
                self.send_header('Access-Control-Allow-Headers', 'Authorization, Content-Type')
                self.end_headers()
                pvinfo={}
                try:
                    if path[1]=='ISrc-010:PBI-BCM-001:AI4-Compressed-Mean':
                        myPV=epics.PV('ISrc-010:PBI-BCM-001:AI4-Compressed', auto_monitor=True)
                        mean_value=myPV.value.mean()
                        tmp_info = myPV.info.split('\n')
                        tmp_info[0]='== ISrc-010:PBI-BCM-001:AI4-Compressed-Mean  (time_double) =='
                        tmp_info[1]='   value      = '+str(mean_value)
                        tmp_info[2]="   char_value = '"+str(mean_value)+"'"
                        tmp_info[3]='   count      = 1'
                        tmp_info[4]='   nelm       = 1'
                        new_info=''
                        for i in tmp_info:
                            new_info+=i
                            new_info+='\n'
                        new_info=new_info[:-1]
                        myPV.clear_auto_monitor()
                        myPV.disconnect()
                        pvinfo['info']=new_info
                        pvinfo['archiver']="pvName = ISrc-010:PBI-BCM-001:AI4-Compressed-Mean\nstatus = Not being archived\n"
                        pvinfo['live_view']="ISrc-010:PBI-BCM-001:AI4-Compressed"
                        self.wfile.write(bytes(json.dumps(pvinfo), "utf-8"))
                        return
                    if path[1]=='ISrc-010:PBI-BCM-001:AI4-Compressed-Max':
                        myPV=epics.PV('ISrc-010:PBI-BCM-001:AI4-Compressed', auto_monitor=True)
                        max_value=myPV.value.max()
                        tmp_info = myPV.info.split('\n')
                        tmp_info[0]='== ISrc-010:PBI-BCM-001:AI4-Compressed-Max  (time_double) =='
                        tmp_info[1]='   value      = '+str(max_value)
                        tmp_info[2]="   char_value = '"+str(max_value)+"'"
                        tmp_info[3]='   count      = 1'
                        tmp_info[4]='   nelm       = 1'
                        new_info=''
                        for i in tmp_info:
                            new_info+=i
                            new_info+='\n'
                        new_info=new_info[:-1]
                        myPV.clear_auto_monitor()
                        myPV.disconnect()
                        pvinfo['info']=new_info
                        pvinfo['archiver']="pvName = ISrc-010:PBI-BCM-001:AI4-Compressed-Max\nstatus = Not being archived\n"
                        pvinfo['live_view']="ISrc-010:PBI-BCM-001:AI4-Compressed"
                        self.wfile.write(bytes(json.dumps(pvinfo), "utf-8"))
                        return

                    myPV=epics.PV(path[1])
                    pvinfo['info']=myPV.info
                    myPV.clear_auto_monitor()
                    myPV.disconnect()
                    archiver_stat = ''
                    archiver_dict=archiver.get_pv_status(pv=path[1])[0]
                    for i in archiver_dict:
                        archiver_stat = archiver_stat + i + ' = '+ archiver_dict[i]+'\n'
                    pvinfo['archiver']=archiver_stat
                    pvinfo['live_view']='http://live.pos.esss.lu.se/live_view?PV='+path[1]
                    if archiver_dict['status'] == 'Being archived':
                        pvinfo['link']='http://plot.pos.esss.lu.se/retrieval/ui/viewer/archViewer.html?pv='+path[1]
                    self.wfile.write(bytes(json.dumps(pvinfo), "utf-8"))
                    return
                except:
                    pvinfo['info']="Something wrong here, please retry"
                    pvinfo['archiver']="Something wrong here, please retry"
                    self.wfile.write(bytes(json.dumps(pvinfo), "utf-8"))
                    return
            return

        self.wfile.write(bytes('The server is running', "utf-8"))
        return

try:
    epicsThread = epicsQuery(stop_signal)
    epicsThread.start()
    server = HTTPServer(('', PORT_NUMBER), myHandler)
    print ('Started httpserver on port ' , PORT_NUMBER)
    server.serve_forever()

except KeyboardInterrupt:
    print ('\n^C received, shutting down the web server')
    server.socket.close()
    stop_signal.set()
